@@ -17,14 +17,14 @@ end
 ---@param uri uri
 ---@param args parser.object
 ---@param removeGeneric true?
----@return table<string, vm.node>
+---@return table<string, vm.node>?
 function mt:resolve(uri, args, removeGeneric)
     if not args then
         return nil
     end
     local resolved = {}
 
-    ---@param object parser.object
+    ---@param object vm.node.object
     ---@param node   vm.node
     local function resolve(object, node)
         if object.type == 'doc.generic.name' then
@@ -33,6 +33,7 @@ function mt:resolve(uri, args, removeGeneric)
                 -- 'number' -> `T`
                 for n in node:eachObject() do
                     if n.type == 'string' then
+                        ---@cast n parser.object
                         local type = vm.declareGlobal('type', n[1], guide.getUri(n))
                         resolved[key] = vm.createNode(type, resolved[key])
                     end
@@ -57,6 +58,7 @@ function mt:resolve(uri, args, removeGeneric)
                 end
                 if n.type == 'global' and n.cate == 'type' then
                     -- ---@field [integer]: number -> T[]
+                    ---@cast n vm.global
                     vm.getClassFields(uri, n, vm.declareGlobal('type', 'integer'), false, function (field)
                         resolve(object.node, vm.compileNode(field.extends))
                     end)
@@ -67,23 +69,37 @@ function mt:resolve(uri, args, removeGeneric)
             for _, ufield in ipairs(object.fields) do
                 local ufieldNode = vm.compileNode(ufield.name)
                 local uvalueNode = vm.compileNode(ufield.extends)
-                if ufieldNode:get(1).type == 'doc.generic.name' and uvalueNode:get(1).type == 'doc.generic.name' then
+                local firstField = ufieldNode:get(1)
+                local firstValue = uvalueNode:get(1)
+                if not firstField or not firstValue then
+                    goto CONTINUE
+                end
+                if firstField.type == 'doc.generic.name' and firstValue.type == 'doc.generic.name' then
                     -- { [number]: number} -> { [K]: V }
                     local tfieldNode = vm.getTableKey(uri, node, 'any')
                     local tvalueNode = vm.getTableValue(uri, node, 'any')
-                    resolve(ufieldNode:get(1), tfieldNode)
-                    resolve(uvalueNode:get(1), tvalueNode)
+                    if tfieldNode then
+                        resolve(firstField, tfieldNode)
+                    end
+                    if tvalueNode then
+                        resolve(firstValue, tvalueNode)
+                    end
                 else
                     if ufieldNode:get(1).type == 'doc.generic.name' then
                         -- { [number]: number}|number[] -> { [K]: number }
                         local tnode = vm.getTableKey(uri, node, uvalueNode)
-                        resolve(ufieldNode:get(1), tnode)
+                        if tnode then
+                            resolve(firstField, tnode)
+                        end
                     elseif uvalueNode:get(1).type == 'doc.generic.name' then
                         -- { [number]: number}|number[] -> { [number]: V }
                         local tnode = vm.getTableValue(uri, node, ufieldNode)
-                        resolve(uvalueNode:get(1), tnode)
+                        if tnode then
+                            resolve(firstValue, tnode)
+                        end
                     end
                 end
+                ::CONTINUE::
             end
         end
     end
@@ -102,6 +118,7 @@ function mt:resolve(uri, args, removeGeneric)
             if obj.type == 'doc.type.table'
             or obj.type == 'doc.type.function'
             or obj.type == 'doc.type.array' then
+                ---@cast obj parser.object
                 local hasGeneric
                 guide.eachSourceType(obj, 'doc.generic.name', function (src)
                     hasGeneric = true
@@ -111,7 +128,7 @@ function mt:resolve(uri, args, removeGeneric)
                     goto CONTINUE
                 end
             end
-            local view = vm.viewObject(obj)
+            local view = vm.viewObject(obj, uri)
             if view then
                 knownTypes[view] = true
             end
@@ -122,21 +139,31 @@ function mt:resolve(uri, args, removeGeneric)
 
     -- remove un-generic type
     ---@param argNode vm.node
+    ---@param sign vm.node
     ---@param knownTypes table<string, true>
     ---@return vm.node
-    local function buildArgNode(argNode, knownTypes)
+    local function buildArgNode(argNode, sign, knownTypes)
         local newArgNode = vm.createNode()
+        local needRemoveNil = sign:hasFalsy()
         for n in argNode:eachObject() do
-            if argNode:hasFalsy() then
-                goto CONTINUE
+            if needRemoveNil then
+                if n.type == 'nil' then
+                    goto CONTINUE
+                end
+                if n.type == 'global' and n.cate == 'type' and n.name == 'nil' then
+                    goto CONTINUE
+                end
             end
-            local view = vm.viewObject(n)
+            local view = vm.viewObject(n, uri)
             if knownTypes[view] then
                 goto CONTINUE
             end
             newArgNode:merge(n)
             ::CONTINUE::
         end
+        if not needRemoveNil and argNode:isOptional() then
+            newArgNode:addOptional()
+        end
         return newArgNode
     end
 
@@ -158,7 +185,7 @@ function mt:resolve(uri, args, removeGeneric)
         local argNode = vm.compileNode(arg)
         local knownTypes, genericNames = getSignInfo(sign)
         if not isAllResolved(genericNames) then
-            local newArgNode = buildArgNode(argNode, knownTypes)
+            local newArgNode = buildArgNode(argNode,sign, knownTypes)
             for n in sign:eachObject() do
                 resolve(n, newArgNode)
             end
